#include <config.h>

#include <dune/grid/albertagrid.hh>
#include <dune/grid/test/gridcheck.hh>
#include <dune/grid/test/checkintersectionit.hh>

#include <dune/subgrid/subgrid.hh>

using namespace Dune;

int main (int argc, char *argv[]) try
{
    typedef Dune::AlbertaGrid<2> GridType;

    typedef GridType::Codim<0>::LevelIterator HostElementIterator;
    typedef SubGrid<2,GridType>::Codim<0>::LevelIterator ElementIterator;

    GridType* gridPtr = NULL;
    GridType& grid = *gridPtr;

    SubGrid<2,GridType> subGrid(grid);

    // create subgrid
    {
        subGrid.createBegin();
        typename GridType::LevelGridView view = grid.levelGridView(0);
        HostElementIterator eIt    = view.begin<0>();
        HostElementIterator eEndIt = view.end<0>();
        for (;eIt!=eEndIt; ++eIt) {
            if (eIt->geometry().corner(0)[0] < 0.5)
                subGrid.insertRaw(*eIt);
        }

        subGrid.createEnd();
    }
    subGrid.report();

    // check subgrid
    std::cout << "Check subgrid<onedgrid>" << std::endl;

    gridcheck(subGrid);
    checkIntersectionIterator(subGrid);

    // refine subgrid
    {
        typename SubGrid<2,GridType>::LevelGridView subView = subGrid.levelGridView(0);
        ElementIterator eIt    = subView.begin<0>();
        ElementIterator eEndIt = subView.end<0>();
        for (;eIt!=eEndIt; ++eIt) {
            if (eIt->geometry().corner(0)[0] < 0.25)
                subGrid.mark(1, *eIt);
        }

        subGrid.preAdapt();
        subGrid.adapt();
        subGrid.postAdapt();
    }

    // check locally refined subgrid
    std::cout << "Check locally refined subgrid<onedgrid>" << std::endl;

    gridcheck(subGrid);
    checkIntersectionIterator(subGrid);

    return 0;
}

// //////////////////////////////////
//   Error handler
// /////////////////////////////////
catch (Exception e) {
    std::cout << e << std::endl;
    return 1;
}
