#include <config.h>

#define DISABLE_DEPRECATED_METHOD_CHECK 1

#include <dune/grid/sgrid.hh>
#include <dune/grid/test/gridcheck.hh>
#include <dune/grid/test/checkintersectionit.hh>


#include <dune/subgrid/subgrid.hh>

using namespace Dune;

template<class GridType>
struct EnableLevelIntersectionIteratorCheck< SubGrid<GridType::dimension,GridType> >
{
  static const bool v = false;
};

// test subgrid for given dimension
template <int dim>
void testDim()
{
    typedef SGrid<dim,dim> GridType;
    int n[dim];
    double h[dim];

    for (int i=0; i<dim; ++i)
    {
        n[i] = 1;
        h[i] = 1.0;
    }

    GridType grid(n,h);

    const int refine = 1;
    for (size_t i=0; i<refine; ++i)
        grid.globalRefine(1);

    typedef typename GridType::template Codim<0>::LevelIterator HostElementIterator;
    typedef typename SubGrid<dim,GridType>::template Codim<0>::LevelIterator ElementIterator;

    SubGrid<dim,GridType> subGrid(grid);

    // create subgrid
#if 0
    {
        subGrid.createBegin();

        typename GridType::LevelGridView view = grid.levelGridView(0);
        HostElementIterator eIt    = view.template begin<0>();
        HostElementIterator eEndIt = view.template end<0>();
        for (;eIt!=eEndIt; ++eIt) {
            if (eIt->geometry().corner(0)[0] < 0.5)
                subGrid.insertRaw(*eIt);
        }

        subGrid.createEnd();
    }
#endif
    {
        subGrid.createBegin();
        subGrid.insertLevel(grid.maxLevel());
        subGrid.createEnd();
        subGrid.setMaxLevelDifference(1);
    }

    subGrid.report();

    // check subgrid
    std::cout << "Check subgrid with dim=" << dim << std::endl;

    gridcheck(subGrid);
    checkIntersectionIterator(subGrid);

    // refine subgrid
    {

        typename SubGrid<dim,GridType>::LevelGridView subView = subGrid.levelGridView(0);
        ElementIterator eIt    = subView.template begin<0>();
        ElementIterator eEndIt = subView.template end<0>();
        for (;eIt!=eEndIt; ++eIt) {
            if (eIt->geometry().corner(0)[0] < 0.25)
                subGrid.mark(1, *eIt);
        }

        subGrid.preAdapt();
        subGrid.adapt();
        subGrid.postAdapt();
    }

    // check locally refined subgrid
    std::cout << "Check locally refined subgrid with dim=" << dim << std::endl;

    gridcheck(subGrid);
    checkIntersectionIterator(subGrid);
}


int main (int argc, char *argv[]) try
{
//     testDim<1>();
    testDim<2>();
    testDim<3>();
}
// //////////////////////////////////
//   Error handler
// /////////////////////////////////
 catch (Exception e) {

    std::cout << e << std::endl;
    return 1;
 }
